#ifndef MEMPOOL_H
#define MEMPOOL_H

/*
	Manages a pool of pre-allocated memory as if it was the heap.
	Calls to mempool_alloc,mempool_free behave exactly like malloc/free
	only they allocate memory from a user allocated buffer.		
*/


/* header for a block*/
typedef struct
{
	long currupt	: 1; 	/* must be set to 0 */
	long inuse	: 1; 	/* allocated or free block */
	long size		: 30; 	/* block size */
}mp_block_header;


typedef struct
{
	long index;
	long size;
	char inuse;
	char *pool;
}mempool;


/* Organizes the memory pointed too by buffer into a memory pool of variable length blocks
   The memory plus the organizational structures are part of the same buffer, when the buffer
   is deleted the memoey pool plus all allocations are wiped out.	
*/
mempool *mempool_create(char *buffer, long size);


/* Analogous to malloc
*/
char *mempool_malloc(mempool *, long size);

/* Analogous to free
*/
void mempool_free(mempool *, char *ptr);


/* Diagnostic function which dumps out the structure of the memory pool
   to stdout.
 */
void mempool_dump(mempool *);


/* returns how much memory is allocated to this pointer or -1 if the address is invalid*/
long mempool_getsize(mempool *, char *ptr);

/* parities on calloc and realloc */
char *mempool_realloc(mempool *, char *ptr, long new_size);
char *mempool_calloc(mempool *, long size);


#endif
