#ifndef SHMEM_H
#define SHMEM_H

#include "mempool.h"
#include <sys/ipc.h>
#include <sys/types.h>
#include <sys/shm.h>
#include <errno.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <ggi/ggi.h>

/* size of each pool of memory */
#define MEMPOOL_SIZE		32767
/* maximum number of pools that we can use */
#define MAX_NUM_POOLS		32
/* rate to poll for an unlock in usecs */
#define LOCK_POLL_RATE          100


typedef struct
{
	/* handle to the shared memory allocated to this structure */
	int shmid;
	/* used for locking and unlocking memory access */
	int inuse;
	
	/* two visuals, one for drawing and the other for buffering */
	ggi_visual_t 
		screen, /* visual for the screen */
		memvis; /* display-memory */
	
	/* index of the top pool */
	int top;
	/* handles to memory pools residing in shared memory */
	int pools[MAX_NUM_POOLS];

	/* temporary mapped pointers to pools created during a lock and modified, and
                 removed during an unlock*/	
	mempool *m[MAX_NUM_POOLS];

}shmem;

/* for the creating process */
shmem *shmem_create();
void shmem_delete(shmem *self);

/* for user processes */
shmem *shmem_open(int shmid);
void shmem_close(shmem *self);

/* locks memory and maps pointer s to memory pools to the user namespace */
int shmem_aquire(shmem *self, long timeout);
void shmem_release(shmem *self);

/* memory operations */
void shmem_free(shmem *self, char *ptr);
char *shmem_malloc(shmem *self, int size);


#endif